"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var _ = require("lodash");
var urlJoin = require("url-join");
var build_tools_1 = require("@aaa-backend-stack/build-tools");
var logger_1 = require("@aaa-backend-stack/logger");
var logger = logger_1.getChildLogger("@aaa-backend-stack/file-storage-s3");
var file_storage_1 = require("@aaa-backend-stack/file-storage");
var LocalFileStorageService = /** @class */ (function () {
    function LocalFileStorageService(config) {
        this.config = config;
        if (_.isEmpty(this.config.baseDirectory)) {
            throw new Error("Missing base directory for local file storage");
        }
        if (_.isEmpty(this.config.publicBaseUrl)) {
            throw new Error("Missing public base URL for local file storage");
        }
    }
    LocalFileStorageService.prototype.writeFile = function (filePath, fileContent, fileContentType, options) {
        if (options === void 0) { options = null; }
        return __awaiter(this, void 0, void 0, function () {
            var content, fullFilePath, fullDirectoryPath, err_1, err_2, op, err_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (_.isNil(options)) {
                            options = file_storage_1.DEFAULT_WRITE_FILE_OPTIONS;
                        }
                        else {
                            options = _.defaults(options, file_storage_1.DEFAULT_WRITE_FILE_OPTIONS);
                        }
                        content = fileContent;
                        if (!options.fileContentAsLocalPath) return [3 /*break*/, 2];
                        return [4 /*yield*/, build_tools_1.FS_EXTRA.readFile(fileContent, options.encoding)];
                    case 1:
                        content = _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!this.isReadStream(fileContent)) return [3 /*break*/, 4];
                        return [4 /*yield*/, new Promise(function (resolve, reject) {
                                var stream = fileContent;
                                var tmpContent = "";
                                stream.setEncoding(options.encoding);
                                stream.on("data", function (chunk) {
                                    tmpContent += chunk;
                                });
                                stream.on("end", function () {
                                    resolve(tmpContent);
                                });
                                stream.on("error", function (err) {
                                    reject(err);
                                });
                            })];
                    case 3:
                        content = _a.sent();
                        _a.label = 4;
                    case 4:
                        fullFilePath = this.getFullFilePath(filePath);
                        fullDirectoryPath = path.dirname(fullFilePath);
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 7, , 8]);
                        return [4 /*yield*/, build_tools_1.FS_EXTRA.ensureDir(fullDirectoryPath)];
                    case 6:
                        _a.sent();
                        return [3 /*break*/, 8];
                    case 7:
                        err_1 = _a.sent();
                        logger.error({ err: err_1, fullDirectoryPath: fullDirectoryPath }, "LocalFileStorageService.writeFile: failed to ensure full directory path exists");
                        throw new Error("Failed to create directory");
                    case 8:
                        if (!!options.overwrite) return [3 /*break*/, 12];
                        _a.label = 9;
                    case 9:
                        _a.trys.push([9, 11, , 12]);
                        return [4 /*yield*/, this.fileExists(filePath)];
                    case 10:
                        if (_a.sent()) {
                            throw new Error("File already exists");
                        }
                        return [3 /*break*/, 12];
                    case 11:
                        err_2 = _a.sent();
                        logger.error({ err: err_2, filePath: filePath, fileContentType: fileContentType, options: options }, "LocalFileStorageService.writeFile: failed to check whether file exists in local file system");
                        throw err_2;
                    case 12:
                        _a.trys.push([12, 15, , 16]);
                        op = build_tools_1.FS_EXTRA.writeFile(fullFilePath, content, { encoding: options.encoding });
                        if (!options.awaitOperation) return [3 /*break*/, 14];
                        return [4 /*yield*/, op];
                    case 13:
                        _a.sent();
                        _a.label = 14;
                    case 14: return [3 /*break*/, 16];
                    case 15:
                        err_3 = _a.sent();
                        logger.error({ err: err_3, fullFilePath: fullFilePath, fileContentType: fileContentType, options: options }, "LocalFileStorageService.writeFile: failed to write file to local file system");
                        // Rethrow only if we're awaiting operation, will be uncaught exception otherwise
                        if (options.awaitOperation) {
                            throw err_3;
                        }
                        return [2 /*return*/];
                    case 16: return [2 /*return*/, {
                            name: _.last(filePath.split("/")),
                            filePath: filePath,
                            publicUrl: this.getPublicUrl(filePath)
                        }];
                }
            });
        });
    };
    LocalFileStorageService.prototype.readFile = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var filePath, options, fullFilePath;
            return __generator(this, function (_a) {
                if (_.isEmpty(args)) {
                    throw new Error("Missing arguments");
                }
                if (_.isString(args[0])) {
                    filePath = args[0];
                }
                else if (_.isObject(args[0])) {
                    filePath = args[0].filePath;
                }
                else {
                    throw new Error("Missing arguments");
                }
                if (args.length >= 2 && !_.isNil(args[1])) {
                    options = _.defaults(args[1], file_storage_1.DEFAULT_READ_FILE_OPTIONS);
                }
                else {
                    options = file_storage_1.DEFAULT_READ_FILE_OPTIONS;
                }
                fullFilePath = this.getFullFilePath(filePath);
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        fs.readFile(fullFilePath, function (err, data) {
                            if (!_.isNil(err)) {
                                if (err.code === "ENOENT" && options.ignoreMissing) {
                                    resolve(null);
                                    return;
                                }
                                logger.error({ err: err, fullFilePath: fullFilePath, options: options }, "LocalFileStorageService.readFile: failed to read file from local file system");
                                reject(err);
                                return;
                            }
                            resolve(data);
                        });
                    })];
            });
        });
    };
    LocalFileStorageService.prototype.deleteFile = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var filePath, options, fullFilePath, op, err_4;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (_.isEmpty(args)) {
                            throw new Error("Missing arguments");
                        }
                        if (_.isString(args[0])) {
                            filePath = args[0];
                        }
                        else if (_.isObject(args[0])) {
                            filePath = args[0].filePath;
                        }
                        else {
                            throw new Error("Missing arguments");
                        }
                        if (args.length >= 2 && !_.isNil(args[1])) {
                            options = _.defaults(args[1], file_storage_1.DEFAULT_DELETE_FILE_OPTIONS);
                        }
                        else {
                            options = file_storage_1.DEFAULT_DELETE_FILE_OPTIONS;
                        }
                        fullFilePath = this.getFullFilePath(filePath);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        op = build_tools_1.FS_EXTRA.unlink(fullFilePath);
                        if (!options.awaitOperation) return [3 /*break*/, 3];
                        return [4 /*yield*/, op];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [3 /*break*/, 5];
                    case 4:
                        err_4 = _a.sent();
                        if (err_4.code === "ENOENT" && options.ignoreMissing) {
                            return [2 /*return*/];
                        }
                        logger.error({ err: err_4, fullFilePath: fullFilePath, options: options }, "LocalFileStorageService.deleteFile: failed to delete file from local file system");
                        // Rethrow only if we're awaiting operation, will be uncaught exception otherwise
                        if (options.awaitOperation) {
                            throw err_4;
                        }
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    LocalFileStorageService.prototype.deleteFiles = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var filePaths, isFilePaths, isFileInfos, options, op, err_5;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (_.isEmpty(args)) {
                            throw new Error("Missing arguments");
                        }
                        isFilePaths = _.some(args[0], _.isString);
                        isFileInfos = _.some(args[0], _.isObject);
                        if (isFilePaths) {
                            filePaths = args[0];
                        }
                        else if (isFileInfos) {
                            filePaths = args[0].map(function (fi) { return fi.filePath; });
                        }
                        else {
                            throw new Error("Missing arguments");
                        }
                        if (args.length >= 2 && !_.isNil(args[1])) {
                            options = _.defaults(args[1], file_storage_1.DEFAULT_DELETE_FILE_OPTIONS);
                        }
                        else {
                            options = file_storage_1.DEFAULT_DELETE_FILE_OPTIONS;
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        op = Promise.map(filePaths, function (filePath) { return __awaiter(_this, void 0, void 0, function () {
                            var err_6;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        _a.trys.push([0, 2, , 3]);
                                        return [4 /*yield*/, this.deleteFile(filePath, options)];
                                    case 1:
                                        _a.sent();
                                        return [3 /*break*/, 3];
                                    case 2:
                                        err_6 = _a.sent();
                                        logger.error({ err: err_6, filePath: filePath, options: options }, "LocalFileStorageService.deleteFiles: failed to delete files from local file system");
                                        return [3 /*break*/, 3];
                                    case 3: return [2 /*return*/];
                                }
                            });
                        }); });
                        if (!options.awaitOperation) return [3 /*break*/, 3];
                        return [4 /*yield*/, op];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [3 /*break*/, 5];
                    case 4:
                        err_5 = _a.sent();
                        logger.error({ err: err_5, options: options }, "LocalFileStorageService.deleteFiles: failed to delete files from local file system");
                        // Rethrow only if we're awaiting operation, will be uncaught exception otherwise
                        if (options.awaitOperation) {
                            throw err_5;
                        }
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    LocalFileStorageService.prototype.fileExists = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var filePath, fullFilePath, fileStats, err_7;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (_.isEmpty(args)) {
                            throw new Error("Missing arguments");
                        }
                        if (_.isString(args[0])) {
                            filePath = args[0];
                        }
                        else if (_.isObject(args[0])) {
                            filePath = args[0].filePath;
                        }
                        else {
                            throw new Error("Missing arguments");
                        }
                        fullFilePath = this.getFullFilePath(filePath);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, build_tools_1.FS_EXTRA.stat(fullFilePath)];
                    case 2:
                        fileStats = _a.sent();
                        return [2 /*return*/, fileStats.isFile()];
                    case 3:
                        err_7 = _a.sent();
                        if (err_7.code !== "ENOENT") {
                            logger.error({ err: err_7, fullFilePath: fullFilePath }, "LocalFileStorageService.fileExists: failed to check if file exists in local file system");
                        }
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/, false];
                }
            });
        });
    };
    LocalFileStorageService.prototype.fileInfo = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return __awaiter(this, void 0, void 0, function () {
            var filePath;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (_.isEmpty(args)) {
                            throw new Error("Missing arguments");
                        }
                        if (_.isString(args[0])) {
                            filePath = args[0];
                        }
                        else if (_.isObject(args[0])) {
                            filePath = args[0].filePath;
                        }
                        else {
                            throw new Error("Missing arguments");
                        }
                        return [4 /*yield*/, this.fileExists(filePath)];
                    case 1:
                        if (!(_a.sent())) {
                            return [2 /*return*/, null];
                        }
                        return [2 /*return*/, {
                                name: _.last(filePath.split("/")),
                                filePath: filePath,
                                publicUrl: this.getPublicUrl(filePath)
                            }];
                }
            });
        });
    };
    LocalFileStorageService.prototype.listFiles = function (directory) {
        return __awaiter(this, void 0, void 0, function () {
            var fullDirectoryPath, files, err_8;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        fullDirectoryPath = path.resolve(this.config.baseDirectory, directory);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, build_tools_1.FS_EXTRA.readdir(fullDirectoryPath)];
                    case 2:
                        files = _a.sent();
                        return [2 /*return*/, files.map(function (file) {
                                return {
                                    name: file,
                                    filePath: path.join(fullDirectoryPath, file),
                                    publicUrl: _this.getPublicUrl(path.join(directory, file))
                                };
                            })];
                    case 3:
                        err_8 = _a.sent();
                        logger.error({ err: err_8, fullDirectoryPath: fullDirectoryPath }, "LocalFileStorageService.listFiles: failed to list files in local file system");
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/, []];
                }
            });
        });
    };
    LocalFileStorageService.prototype.isAccessible = function (rethrow) {
        if (rethrow === void 0) { rethrow = false; }
        return __awaiter(this, void 0, void 0, function () {
            var err_9;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, build_tools_1.FS_EXTRA.access(this.config.baseDirectory, fs.constants.W_OK)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, true];
                    case 2:
                        err_9 = _a.sent();
                        logger.error({ err: err_9 }, "LocalFileStorageService.isAccessible: received error while checking base directory access");
                        if (rethrow) {
                            throw err_9;
                        }
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/, false];
                }
            });
        });
    };
    LocalFileStorageService.prototype.getFullFilePath = function (filePath) {
        return path.resolve(path.join(this.config.baseDirectory, filePath));
    };
    LocalFileStorageService.prototype.getPublicUrl = function (filePath) {
        return urlJoin(this.config.publicBaseUrl, filePath);
    };
    LocalFileStorageService.prototype.isReadStream = function (content) {
        return content !== null &&
            typeof content === "object" &&
            typeof content.pipe === "function" &&
            content.readable !== false &&
            typeof content._read === "function" &&
            typeof content._readableState === "object";
    };
    return LocalFileStorageService;
}());
exports.LocalFileStorageService = LocalFileStorageService;
//# sourceMappingURL=LocalFileStorageService.js.map